@@ -11,23 +11,43 @@ from ..helper import helper
 
 
 class Reader:
+    """
+    Parent class for all readers. Subclasses need to implement their call method.
+    """
+
     def __init__(self):
         pass
 
     def __call__(self, row, path_to_image):
-        pass
+        raise NotImplementedError
 
 
 class ReadTestImageV2(Reader):
+    """
+    TODO
+    """
+
     def __init__(
         self,
-        image_size=100,
-        to_hash_added=0,
-        num_of_circles=10,
-        return_mask=False,
-        return_coordinats=False,
-        return_bbox=False,
+        image_size: int = 100,
+        to_hash_added:int = 0,
+        num_of_circles: int = 10,
+        return_mask: bool = False,
+        return_coordinats: bool = False,
+        return_bbox: bool = False
     ):
+        """
+        TODO
+
+        Arguments:
+            image_size: TODO
+            to_hash_added: TODO
+            num_of_circles: TODO
+            return_mask: TODO
+            return_coordinats: TODO
+            return_bbox: TODO
+        """
+
         super().__init__()
         self.img_size = image_size
         self.to_hash_added = to_hash_added
@@ -36,10 +56,17 @@ class ReadTestImageV2(Reader):
         self.return_bbox = return_bbox
         self.num_of_circles = num_of_circles
 
-    def create_test_image(
-        self,
-        row,
-    ):
+    def create_test_image(self,row: int) -> dict:
+        """
+        TODO
+
+        Arguments:
+            row: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         img = np.ones([self.img_size, self.img_size, 3], dtype=np.uint8) * 255
 
         hash_of_row = row["__hash_of_row__"] + self.to_hash_added
@@ -92,7 +119,18 @@ class ReadTestImageV2(Reader):
             result.pop("bbox")
         return result
 
-    def __call__(self, row_or_df, path_to_image):
+    def __call__(self, row_or_df, path_to_image: str):
+        """
+        TODO
+
+        Arguments:
+            row_or_df: TODO
+            path_to_image: Absolute path to to image.
+
+        Returns:
+            TODO: TODO
+        """
+        
         if isinstance(row_or_df, pd.DataFrame):
             imgs = {}
 
@@ -109,14 +147,30 @@ class ReadTestImageV2(Reader):
 
 
 class ReadTestImage(Reader):
+    """
+    TODO
+    """
+
     def __init__(
         self,
-        image_size=100,
-        to_hash_added=0,
-        return_mask=False,
-        return_coordinats=False,
-        return_bbox=False,
+        image_size: int = 100,
+        to_hash_added: int = 0,
+        return_mask: bool = False,
+        return_coordinats: bool = False,
+        return_bbox: bool = False
     ):
+        """
+        TODO
+
+        Arguments:
+            image_size: TODO
+            to_hash_added: TODO
+            return_mask: TODO
+            return_coordinats: TODO
+            return_bbox: TODO
+
+        """
+
         super().__init__()
         self.img_size = image_size
         self.to_hash_added = to_hash_added
@@ -128,6 +182,16 @@ class ReadTestImage(Reader):
         self,
         row,
     ):
+        """
+        TODO
+
+        Arguments:
+            row: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         img = np.ones([self.img_size, self.img_size, 3], dtype=np.uint8) * 255
 
         hash_of_row = row["__hash_of_row__"] + self.to_hash_added
@@ -188,6 +252,17 @@ class ReadTestImage(Reader):
         return result
 
     def __call__(self, row_or_df, path_to_image):
+        """
+        TODO
+
+        Arguments:
+            row_or_df: TODO
+            path_to_image: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         if isinstance(row_or_df, pd.DataFrame):
             imgs = {}
 
@@ -204,6 +279,18 @@ class ReadTestImage(Reader):
 
 
 def expression_to_string_value(expression, interpreter, row):
+    """
+    TODO
+
+    Arguments:
+        expression: TODO
+        interpreter: TODO
+        row: TODO
+
+    Returns:
+        TODO: TODO
+    """
+
     if expression in row:
         return row[expression]
     elif callable(expression):
@@ -215,14 +302,29 @@ def expression_to_string_value(expression, interpreter, row):
 
 
 class ReadFromImageFile(Reader):
+    """
+    TODO
+    """
+    
     def __init__(
         self,
         expression,
         interpreter=helper.read_from_row,
         return_image_size=None,
-        x_column="__x__",
-        y_column="__y__",
+        x_column: str = "__x__",
+        y_column: str = "__y__",
     ):
+        """
+        TODO
+
+        Arguments:
+            expression: TODO
+            interpreter: TODO
+            return_image_size: TODO
+            x_column: TODO
+            y_column: TODO
+        """
+
         super().__init__()
         self.expression = expression
         self.interpreter = interpreter
@@ -231,6 +333,18 @@ class ReadFromImageFile(Reader):
         self.y_column = y_column
 
     def crop(self, img, x, y):
+        """
+        TODO
+
+        Arguments:
+            img: TODO
+            x: TODO
+            y: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         shifted_x, shifted_y = 0, 0
         start_x = int(x - self.return_image_size / 2)
         if start_x < 0:
@@ -244,20 +358,35 @@ class ReadFromImageFile(Reader):
         end_y = start_y + self.return_image_size
         img = img[start_x:end_x, start_y:end_y]
         if img.shape[0] != self.return_image_size or img.shape[1] != self.return_image_size:
-            img_black_boarder = np.zeros([self.return_image_size,self.return_image_size,img.shape[2]], dtype=img.dtype)
-            img_black_boarder[shifted_x:shifted_x+img.shape[0],shifted_y:shifted_y+img.shape[1]] = img
+            img_black_boarder = np.zeros(
+                [self.return_image_size, self.return_image_size, img.shape[2]], dtype=img.dtype
+            )
+            img_black_boarder[
+                shifted_x : shifted_x + img.shape[0], shifted_y : shifted_y + img.shape[1]
+            ] = img
             return img_black_boarder
         else:
             return img
 
     def load_from_row(self, row, path_to_image):
+        """
+        TODO
+
+        Arguments:
+            row: TODO
+            path_to_image: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         path = expression_to_string_value(self.expression, self.interpreter, row)
         path_to_file = os.path.join(path_to_image, path)
         # img = plt.imread(path_to_file)
         img = np.asarray(Image.open(path_to_file))
         if len(img.shape) == 3 and img.shape[-1] == 4:
             img = img[..., :3]
-        if self.return_image_size is not None:
+        if self.return_image_size:
             if self.x_column in row:
                 x = row[self.x_column]
             else:
@@ -274,11 +403,33 @@ class ReadFromImageFile(Reader):
         return img
 
     def get_image_size(self, row, path_to_image):
+        """
+        TODO
+
+        Arguments:
+            row: TODO
+            path_to_image: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         path = expression_to_string_value(self.expression, self.interpreter, row)
         path_to_file = os.path.join(path_to_image, path)
         return imagesize.get(path_to_file)
 
     def __call__(self, row_or_df, path_to_image):
+        """
+        TODO
+
+        Arguments:
+            row_or_df: TODO
+            path_to_image: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         if isinstance(row_or_df, pd.DataFrame):
             imgs = []
             for _, row in row_or_df.iterrows():
@@ -289,16 +440,34 @@ class ReadFromImageFile(Reader):
 
 
 class ReadFromWSI(Reader):
+    """
+    TODO
+    """
+    
     def __init__(
         self,
         return_image_size,
         expression,
         interpreter=helper.read_from_row,
-        mpp=0.5,
-        x_column="__x__",
-        y_column="__y__",
-        use_center_coordinates=True,
+        mpp: float = 0.5,
+        x_column: str = "__x__",
+        y_column: str = "__y__",
+        use_center_coordinates: bool = True
     ):
+        """
+        TODO
+
+        Arguments:
+            return_image_size: TODO
+            expression: TODO
+            interpreter: TODO
+            mpp: TODO
+            x_column: TODO
+            y_column: TODO
+            use_center_coordinates: TODO
+
+        """
+
         super().__init__()
         if isinstance(return_image_size, int):
             self.return_image_size = (return_image_size, return_image_size)
@@ -312,6 +481,17 @@ class ReadFromWSI(Reader):
         self.use_center_coordinates = use_center_coordinates
 
     def load_from_row(self, row, path_to_image):
+        """
+        TODO
+
+        Arguments:
+            row: TODO
+            path_to_image: Absolute path to image.
+
+        Returns:
+            TODO: TODO
+        """
+
         path = expression_to_string_value(self.expression, self.interpreter, row)
         path_to_file = os.path.join(path_to_image, path)
 
@@ -340,6 +520,19 @@ class ReadFromWSI(Reader):
         return patch
 
     def read_thumbnail(self, row, path_to_image, mpp, return_min_mpp=False):
+        """
+        TODO
+
+        Arguments:
+            row: TODO
+            path_to_image: TODO
+            mpp: TODO
+            return_min_mpp: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         path = expression_to_string_value(self.expression, self.interpreter, row)
         path_to_file = os.path.join(path_to_image, path)
 
@@ -355,6 +548,17 @@ class ReadFromWSI(Reader):
             return wsi_thumb
 
     def __call__(self, row_or_df, path_to_image):
+        """
+        TODO
+
+        Arguments:
+            row_or_df: TODO
+            path_to_image: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         if isinstance(row_or_df, pd.DataFrame):
             imgs = []
             for _, row in row_or_df.iterrows():
@@ -365,15 +569,39 @@ class ReadFromWSI(Reader):
 
 
 class ReadValueFromCSV(Reader):
+    """
+    TODO
+    """
+    
     def __init__(self, expression, interpreter=helper.read_from_row, encoded_values=None):
+        """
+        TODO
+
+        Arguments:
+            expression: TODO
+            interpreter: TODO
+            encoded_values: TODO
+
+        """
+
         super().__init__()
         self.expression = expression
         self.interpreter = interpreter
         self.encoded_values = encoded_values
 
     def load_from_row(self, row):
+        """
+        TODO
+
+        Arguments:
+            row: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         loaded_value = expression_to_string_value(self.expression, self.interpreter, row)
-        if self.encoded_values is not None:
+        if self.encoded_values:
             if loaded_value in self.encoded_values:
                 return self.encoded_values.index(loaded_value)
             else:
@@ -381,6 +609,17 @@ class ReadValueFromCSV(Reader):
         return loaded_value
 
     def __call__(self, row_or_df, path_to_image):
+        """
+        TODO
+
+        Arguments:
+            row_or_df: TODO
+            path_to_image: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         if isinstance(row_or_df, pd.DataFrame):
             values = []
             for _, row in row_or_df.iterrows():
