@@ -1,44 +1,88 @@
+import json
 import os
 import re
+from typing import Optional as O
 
 import pandas as pd
-import json
 
 
 class DataFrameCreator:
+    """
+    Parent class for all dataframes. Subclasses need to implement their call method.
+    """
+
     def __init__(self):
         pass
 
-    def __call__(self, path_to_dataset):
-        pass
+    def __call__(self, path_to_dataset: str):
+        raise NotImplementedError
 
 
 class CreateDFDummy(DataFrameCreator):
-    def __init__(self, num_of_entries=100):
+    """
+    Creates a dummy DataFrame.
+    """
+
+    def __init__(self, num_of_entries: int = 100):        
+        """
+        TODO
+
+        Arguments:
+            num_of_entries: TODO
+        """
+
         super().__init__()
         self.num_of_entries = num_of_entries
 
-    def __call__(self, path_to_dataset):
+    def __call__(self, path_to_dataset: str):
+        """
+        TODO
+
+        Arguments:
+            path_to_dataset: TODO
+        """
         df = pd.DataFrame({"id": range(self.num_of_entries)})
         df["fold"] = df.apply(lambda row: row["id"] % 10, 1)
         df["is_even"] = df.apply(lambda row: (row["id"] % 2) == 0, 1)
         return df
 
-    
+
 class CreateDFFromJSON(DataFrameCreator):
-    def __init__(self, path_to_json, key_to_use=None):
+    """
+    Creates a DataFrame from a JSON file.
+    """
+
+    def __init__(self, path_to_json: str, key_to_use: O[str] = None):
+        """
+        TODO
+
+        Arguments:
+            path_to_json: TODO
+            key_to_use: TODO
+        """
+
         super().__init__()
         self.path_to_json = path_to_json
         self.key_to_use = key_to_use
 
-    def __call__(self, path_to_dataset):
+    def __call__(self, path_to_dataset: str):
+        """
+        TODO
+
+        Arguments:
+            path_to_dataset: TODO
+
+        Returns:
+            TODO: TODO
+        """
+        
         if isinstance(self.path_to_json, list):
             df = pd.DataFrame()
             for path_to_json in self.path_to_json:
                 path = os.path.join(path_to_dataset, path_to_json)
                 with open(path) as f:
                     json_dict = json.load(f)
-                if self.key_to_use is None:
+                if self.key_to_use:
                     tmp_df = pd.DataFrame(json_dict)
                 else:
                     tmp_df = pd.DataFrame(json_dict[self.key_to_use])
@@ -49,20 +93,41 @@ class CreateDFFromJSON(DataFrameCreator):
             path = os.path.join(path_to_dataset, self.path_to_json)
             with open(path) as f:
                 json_dict = json.load(f)
-            if self.key_to_use is None:
+            if self.key_to_use:
                 return pd.DataFrame(json_dict)
             else:
                 return pd.DataFrame(json_dict[self.key_to_use])
-        
+
 
 class CreateDFFromCSV(DataFrameCreator):
-    def __init__(self, path_to_csv, sep=",", **argv):
+    """
+    Creates a DataFrame from CSV.
+    """
+
+    def __init__(self, path_to_csv: str, sep: str = ",", **argv):
+        """
+        TODO
+
+        Arguments:
+            path_to_csv: TODO
+            sep: TODO
+            **argv: TODO
+        """
         super().__init__()
         self.path_to_csv = path_to_csv
         self.sep = sep
         self.argv = argv
 
-    def __call__(self, path_to_dataset):
+    def __call__(self, path_to_dataset: str):
+        """
+        TODO
+
+        Arguments:
+            path_to_dataset: Absolute path to dataset
+
+        Returns:
+            TODO: TODO
+        """
         if isinstance(self.path_to_csv, list):
             df = pd.DataFrame()
             for path_to_csv in self.path_to_csv:
@@ -77,7 +142,18 @@ class CreateDFFromCSV(DataFrameCreator):
 
 
 class CreateDFFromFolder(DataFrameCreator):
-    def __init__(self, file_of_interest):
+    """
+    Creates DataFrame based on a folder content.
+    """
+
+    def __init__(self, file_of_interest: str):
+        """
+        TODO
+        
+        Arguments:
+            file_of_interest: TODO
+        """
+
         super().__init__()
         self.file_of_interest = file_of_interest
 
@@ -94,7 +170,7 @@ class CreateDFFromFolder(DataFrameCreator):
                 and (var_unnamed_start == 0 or tmp[var_unnamed_start - 1] != "\\")
             ):
                 regex += tmp[: var_unnamed_start + 1]
-                tmp = tmp[var_unnamed_start + 1 :]
+                tmp = tmp[var_unnamed_start + 1:]
                 self.var_names.append("unnamed_" + str(counter_unnamed))
                 counter_unnamed += 1
             elif var_start == -1:  # No Variable could be found
@@ -113,24 +189,34 @@ class CreateDFFromFolder(DataFrameCreator):
                     # self.regex += '(?=.*)'
                     # regex += '(^[^/]*)'
                     regex += "(.*)"
-                    tmp = tmp[var_end + 1 :]
+                    tmp = tmp[var_end + 1:]
         self.regex = re.compile(regex)
 
-    def __call__(self, path_to_dataset):
+    def __call__(self, path_to_dataset: str):
+        """
+        TODO
+
+        Arguments:
+            path_to_dataset: Absolute path to dataset
+
+        Returns:
+            TODO: TODO
+        """
+
         rows = {name: [] for name in self.var_names}
         rows["__path__"] = []
         for root, _, files in os.walk(path_to_dataset):
             for file in files:
-                path = os.path.join(root, file)[len(path_to_dataset) :]
+                path = os.path.join(root, file)[len(path_to_dataset):]
                 if path.startswith("/"):
                     path = path[1:]
                 result = self.regex.search(path)
-                if result is not None:
+                if result:
                     path_as_variable = False
                     row = {}
                     for vname, var in zip(self.var_names, result.groups()):
                         row[vname] = var
-                        if var is not None and var.find("/") != -1:
+                        if var and var.find("/") != -1:
                             path_as_variable = True
                     if path_as_variable is False:
                         for k in row:
