@@ -13,45 +13,88 @@ from skimage.segmentation import slic
 
 
 class DataFrameManipulator:
+    """
+    Parent class for all manipulators. Subclasses need to implement their call method.
+    """
     def __init__(self):
         pass
 
     def __call__(self, df, path_to_image, reader):
         pass
 
-    
+
 class BoundingBoxToCenterCoordinates(DataFrameManipulator):
-    def __init__(self, column_of_bbox, name_of_x_column='__x__', name_of_y_column='__y__'):
+    """
+    TODO
+    """
+    
+    def __init__(self, column_of_bbox: str, name_of_x_column: str = '__x__', name_of_y_column: str = '__y__'):
         self.column_of_bbox = column_of_bbox
         self.name_of_x_column = name_of_x_column
         self.name_of_y_column = name_of_y_column
+
     def __call__(self, df, path_to_image, reader):
         x = []
         y = []
         for _, row in df.iterrows():
-            x.append((row[self.column_of_bbox][1]+row[self.column_of_bbox][3])/2)
-            y.append((row[self.column_of_bbox][0]+row[self.column_of_bbox][2])/2)
+            x.append((row[self.column_of_bbox][1] + row[self.column_of_bbox][3]) / 2)
+            y.append((row[self.column_of_bbox][0] + row[self.column_of_bbox][2]) / 2)
         df[self.name_of_x_column] = x
         df[self.name_of_y_column] = y
         return df
-            
-            
+
+
 class EditDFImageGrid(DataFrameManipulator):
-    def __init__(self, return_image_size, perc_overlapping=0.0, input_im_size=None):
+    """
+    TODO
+    """
+
+    def __init__(self, return_image_size, perc_overlapping: float = 0.0, input_im_size=None):
+        """
+        TODO
+
+        Arguments:
+            return_image_size: TODO
+            perc_overlapping: TODO
+            input_im_size: TODO
+        """
+
         super().__init__()
         self.return_image_size = return_image_size
         self.perc_overlapping = perc_overlapping
         self.input_im_size = input_im_size
 
     def get_points(self, im_size1d):
+        """
+        TODO
+
+        Arguments:
+            im_size1d: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         im_size_center = im_size1d - self.return_image_size
         shift = self.return_image_size * (1 - self.perc_overlapping)
         locations = np.arange(0, im_size_center + 0.0001, shift)
 
         return np.array(locations + (im_size1d - locations[-1]) / 2, dtype=int)
 
-    def __call__(self, df, path_to_image, reader):
-        if self.input_im_size is not None:
+    def __call__(self, df, path_to_image: str, reader):
+        """
+        TODO
+
+        Arguments:
+            df: TODO
+            path_to_image: TODO
+            reader: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
+        if self.input_im_size:
             input_im_size = self.input_im_size
 
         rows = {name: [] for name in list(df)}
@@ -76,6 +119,16 @@ class EditDFImageGrid(DataFrameManipulator):
 
 
 def get_mask(rgb):
+    """
+    TODO
+
+    Arguments:
+        rgb: TODO
+
+    Returns:
+        TODO: TODO
+    """
+
     rgb = np.array(rgb, dtype=float) / 255.0
 
     ihc_hed = rgb2hed(rgb)
@@ -95,9 +148,9 @@ def get_mask(rgb):
         if threshold2 is None:
             if counts[pos + 1] > counts[pos]:
                 threshold2 = positions[pos]
-    if threshold1 is not None:
+    if threshold1:
         threshold = threshold1
-    elif threshold2 is not None:
+    elif threshold2:
         threshold = threshold2
     else:
         threshold = 0.04
@@ -120,13 +173,27 @@ def get_mask(rgb):
 
 
 class EditDFWSIGrid(DataFrameManipulator):
+    """
+    TODO
+    """
+
     def __init__(
         self,
         return_image_size,
-        perc_overlapping=0.0,
-        plt_coordinates=False,
+        perc_overlapping: float = 0.0,
+        plt_coordinates: bool = False,
         fn_thumbnail_to_mask=get_mask,
     ):
+        """
+        TODO
+
+        Arguments:
+            return_image_size: TODO
+            perc_overlapping: TODO
+            plt_coordinates: TODO
+            fn_thumbnail_to_mask: TODO
+ 
+        """
         super().__init__()
         self.return_image_size = return_image_size
         self.perc_overlapping = perc_overlapping
@@ -134,6 +201,19 @@ class EditDFWSIGrid(DataFrameManipulator):
         self.fn_thumbnail_to_mask = fn_thumbnail_to_mask
 
     def __get_coordinates__(self, thumbnail, thumbnail_mpp, return_mpp, min_mpp):
+        """
+        TODO
+
+        Arguments:
+            thumbnail: TODO
+            thumbnail_mpp: TODO
+            return_mpp: TODO
+            min_mpp: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         return_mpp = float(return_mpp)
         thumbnail_mpp = float(thumbnail_mpp)
         # see: https://colab.research.google.com/github/TIA-Lab
@@ -193,6 +273,16 @@ class EditDFWSIGrid(DataFrameManipulator):
         )
 
     def __call__(self, df, path_to_image, reader):
+        """
+        TODO
+
+        Arguments:
+            path_to_image: Absolute path to image.
+            reader: TODO
+
+        Returns:
+            TODO: TODO
+        """
 
         rows = {name: [] for name in list(df)}
         rows["__x__"] = []
@@ -223,10 +313,32 @@ class EditDFWSIGrid(DataFrameManipulator):
 
 
 class DataFrameFilter:
+    """
+    TODO
+    """
+
     def __init__(self, mode):
+        """
+        TODO
+
+        Arguments:
+            mode: TODO
+
+        """
         self.mode = mode
 
     def __call__(self, df, indices):
+        """
+        TODO
+
+        Arguments:
+            df: TODO
+            indices: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         if self.mode == 0:
             return df.loc[indices]
         elif self.mode == 1:
@@ -236,20 +348,61 @@ class DataFrameFilter:
 
 
 class RandomFilter(DataFrameFilter, DataFrameManipulator):
+    """
+    TODO
+    """
+
     def __init__(self, percentage_to_filter, mode=0, seed=None):
+        """
+        TODO
+
+        Arguments:
+            percentage_to_filter: TODO
+            mode: TODO
+            seed: TODO
+
+        """
+
         DataFrameFilter.__init__(self, mode)
         DataFrameManipulator.__init__(self)
         self.percentage_to_filter = percentage_to_filter
         self.seed = seed
 
     def __call__(self, df, path_to_image, reader):
+        """
+        TODO
+
+        Arguments:
+            df: TODO
+            path_to_image: Absolute path to image.
+            reader: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         r = random.Random(self.seed)
         indices = r.sample(list(df.index), int(len(df) * self.percentage_to_filter))
         return DataFrameFilter.__call__(self, df, indices)
 
 
 class RandomFilterByColumnValue(DataFrameFilter, DataFrameManipulator):
+    """
+    TODO
+    """
+
     def __init__(self, column, percentage_to_filter, mode=0, seed=None):
+        """
+        TODO
+
+        Arguments:
+            column: TODO
+            percentage_to_filter: TODO
+            mode: TODO
+            seed: TODO
+
+        """
+
         DataFrameFilter.__init__(self, mode)
         DataFrameManipulator.__init__(self)
         self.column = column
@@ -257,6 +410,17 @@ class RandomFilterByColumnValue(DataFrameFilter, DataFrameManipulator):
         self.seed = seed
 
     def __call__(self, df, path_to_image, reader):
+        """
+        TODO
+
+        Arguments:
+            df: TODO
+            path_to_image: Absolute path to image.
+            reader: TODO
+
+        Returns:
+            TODO: TODO
+        """
         uv = list(df[self.column].unique())
         r = random.Random(self.seed)
         choosen = r.sample(uv, int(len(uv) * self.percentage_to_filter))
@@ -265,23 +429,73 @@ class RandomFilterByColumnValue(DataFrameFilter, DataFrameManipulator):
 
 
 class LambdaFilter(DataFrameFilter, DataFrameManipulator):
+    """
+    TODO
+    """
+
     def __init__(self, lambda_function, mode=0):
+        """
+        TODO
+
+        Arguments:
+            lambda_function: TODO
+            mode: TODO
+
+        """
+
         DataFrameFilter.__init__(self, mode)
         DataFrameManipulator.__init__(self)
         self.lambda_function = lambda_function
 
     def __call__(self, df, path_to_image, reader):
+        """
+        TODO
+
+        Arguments:
+            df: TODO
+            path_to_image: Absolute path to image.
+            reader: TODO
+
+        Returns:
+            TODO: TODO
+        """
         return DataFrameFilter.__call__(self, df, df[df.apply(self.lambda_function, 1)].index)
 
 
 class ColumnFilter(DataFrameFilter, DataFrameManipulator):
+    """
+    TODO
+    """
+
     def __init__(self, name_of_column, allowed_values, mode=0):
+        """
+        TODO
+
+        Arguments:
+            name_of_column: TODO
+            allowed_values: TODO
+            mode: TODO
+
+        """
+
         DataFrameFilter.__init__(self, mode)
         DataFrameManipulator.__init__(self)
         self.name_of_column = name_of_column
         self.allowed_values = allowed_values
 
     def __call__(self, df, path_to_image, reader):
+        """
+        TODO
+
+        Arguments:
+            df: TODO
+            path_to_image: Absolute path to image.
+            reader: TODO
+
+        Returns:
+            TODO: TODO
+        """
+
         if isinstance(self.allowed_values, list):
             return DataFrameFilter.__call__(
                 self, df, df[df[self.name_of_column].isin(self.allowed_values)].index
