@@ -89,3 +89,12 @@ def set_random_seed_with_int(integer):
 
 def read_from_row(row, expression):
     return expression.format(**row)
+
+
+def labels_to_weights(labels):
+    labels = np.array(labels)
+    label_prob = np.zeros_like(labels, dtype=np.float)
+    for l in np.unique(labels):
+        prob = sum(labels == l) / float(len(labels))
+        label_prob[labels == l] = 1 - prob
+    return label_prob
